 /*******************************************************************************\
 * Copyright (c) 2008, Claudio Pica     and Agostino Patella                     *   
 * All rights reserved.                                                          * 
 * Redistribution and use in source and binary forms, with or without            *
 * modification, are permitted provided that the following conditions are met:   *
 *                                                                               *
 * 1)  Redistributions of source code must retain the above copyright notice,    *
 *     this list of conditions and the following disclaimer.                     *
 * 2)  Redistributions in binary form must reproduce the above copyright notice, *
 *     this list of conditions and the following disclaimer in the documentation *
 *     and/or other materials provided with the distribution.                    *
 * 3)  Any publication in any form derived from the use of this software or any  *
 *     modification of it must refer explicitly to the original BSMBench package *
 *     (including the official URL http://github.com/blucini/BSMBench)           *
 *     and cite the following two publications:                                  *
 *       [1] L. Del Debbio, A. Patella, C. Pica, "Higher representations on the  *
 *           lattice: Numerical simulations. SU(2) with adjoint fermions", Phys. *
 * 	  Rev. D81 (2010) 094503, DOI 10.1103/PhysRevD.81.094503                 *
 *       [2] L. Del Debbio, B. Lucini, A. Patella, C. Pica, A. Rago, "Conformal  *
 *       versus confining scenario in SU(2) with adjoint fermions", Phys.Rev.    *
 *       D80 (2009) 074507, DOI 10.1103/PhysRevD.80.074507                       *
 * 4)  The names of the contributors may be used to endorse or promote products  *
 *     derived from this software without specific prior written permission.     *
 *                                                                               *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"   *
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE     *
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE    *
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE     *
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR           *
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF          *
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS      *
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       *
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)       *
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF        *
 * THE POSSIBILITY OF SUCH DAMAGE.                                               *
 \*******************************************************************************/

#ifndef SUN_REPR_FUNC_H
#define SUN_REPR_FUNC_H

#if (NG!=6) 
#error : Mismatch between NG and representation functions!
#endif
#if (NF!=6) 
#error : Mismatch between NF and representation functions!
#endif



 /*****
 *   _FUND_NORM2 = C(FUND)
 *   _FUND_NORM2 = C(REPR)
 */
#define _FUND_NORM2 (+5.000000000000000e-01)
#define _REPR_NORM2 (+5.000000000000000e-01)

 /*****
 *	v COMPLEX matrix 6*6
 *	u complex matrix 6*6
 *	v is the representation of u, as element of SU(6)
 */
 /*****
#define _group_represent(v,u) \
REPR::GROUP_REPRESENT:(v):(u)
 */

 /*****
 *	m COMPLEX matrix 6*6
 *	h real 35-vector
 *	m = sum(A) i*h(A)*rT(A)
 */
#define _algebra_represent(m,h) \
	(m).c[0].re = 0.0; \
	(m).c[0].im = +2.886751345948129e-01*(h).c[13]+2.041241452319315e-01*(h).c[20]+1.581138830084190e-01*(h).c[27]+1.290994448735805e-01*(h).c[34]+5.000000000000000e-01*(h).c[6]; \
	(m).c[1].re = -5.000000000000000e-01*(h).c[0]; \
	(m).c[1].im = +5.000000000000000e-01*(h).c[5]; \
	(m).c[2].re = -5.000000000000000e-01*(h).c[1]; \
	(m).c[2].im = +5.000000000000000e-01*(h).c[11]; \
	(m).c[3].re = -5.000000000000000e-01*(h).c[2]; \
	(m).c[3].im = +5.000000000000000e-01*(h).c[17]; \
	(m).c[4].re = -5.000000000000000e-01*(h).c[3]; \
	(m).c[4].im = +5.000000000000000e-01*(h).c[23]; \
	(m).c[5].re = -5.000000000000000e-01*(h).c[4]; \
	(m).c[5].im = +5.000000000000000e-01*(h).c[29]; \
	(m).c[6].re = +5.000000000000000e-01*(h).c[0]; \
	(m).c[6].im = +5.000000000000000e-01*(h).c[5]; \
	(m).c[7].re = 0.0; \
	(m).c[7].im = +2.886751345948129e-01*(h).c[13]+2.041241452319315e-01*(h).c[20]+1.581138830084190e-01*(h).c[27]+1.290994448735805e-01*(h).c[34]-5.000000000000000e-01*(h).c[6]; \
	(m).c[8].re = -5.000000000000000e-01*(h).c[7]; \
	(m).c[8].im = +5.000000000000000e-01*(h).c[12]; \
	(m).c[9].re = -5.000000000000000e-01*(h).c[8]; \
	(m).c[9].im = +5.000000000000000e-01*(h).c[18]; \
	(m).c[10].re = -5.000000000000000e-01*(h).c[9]; \
	(m).c[10].im = +5.000000000000000e-01*(h).c[24]; \
	(m).c[11].re = -5.000000000000000e-01*(h).c[10]; \
	(m).c[11].im = +5.000000000000000e-01*(h).c[30]; \
	(m).c[12].re = +5.000000000000000e-01*(h).c[1]; \
	(m).c[12].im = +5.000000000000000e-01*(h).c[11]; \
	(m).c[13].re = +5.000000000000000e-01*(h).c[7]; \
	(m).c[13].im = +5.000000000000000e-01*(h).c[12]; \
	(m).c[14].re = 0.0; \
	(m).c[14].im = -5.773502691896257e-01*(h).c[13]+2.041241452319315e-01*(h).c[20]+1.581138830084190e-01*(h).c[27]+1.290994448735805e-01*(h).c[34]; \
	(m).c[15].re = -5.000000000000000e-01*(h).c[14]; \
	(m).c[15].im = +5.000000000000000e-01*(h).c[19]; \
	(m).c[16].re = -5.000000000000000e-01*(h).c[15]; \
	(m).c[16].im = +5.000000000000000e-01*(h).c[25]; \
	(m).c[17].re = -5.000000000000000e-01*(h).c[16]; \
	(m).c[17].im = +5.000000000000000e-01*(h).c[31]; \
	(m).c[18].re = +5.000000000000000e-01*(h).c[2]; \
	(m).c[18].im = +5.000000000000000e-01*(h).c[17]; \
	(m).c[19].re = +5.000000000000000e-01*(h).c[8]; \
	(m).c[19].im = +5.000000000000000e-01*(h).c[18]; \
	(m).c[20].re = +5.000000000000000e-01*(h).c[14]; \
	(m).c[20].im = +5.000000000000000e-01*(h).c[19]; \
	(m).c[21].re = 0.0; \
	(m).c[21].im = -6.123724356957945e-01*(h).c[20]+1.581138830084190e-01*(h).c[27]+1.290994448735805e-01*(h).c[34]; \
	(m).c[22].re = -5.000000000000000e-01*(h).c[21]; \
	(m).c[22].im = +5.000000000000000e-01*(h).c[26]; \
	(m).c[23].re = -5.000000000000000e-01*(h).c[22]; \
	(m).c[23].im = +5.000000000000000e-01*(h).c[32]; \
	(m).c[24].re = +5.000000000000000e-01*(h).c[3]; \
	(m).c[24].im = +5.000000000000000e-01*(h).c[23]; \
	(m).c[25].re = +5.000000000000000e-01*(h).c[9]; \
	(m).c[25].im = +5.000000000000000e-01*(h).c[24]; \
	(m).c[26].re = +5.000000000000000e-01*(h).c[15]; \
	(m).c[26].im = +5.000000000000000e-01*(h).c[25]; \
	(m).c[27].re = +5.000000000000000e-01*(h).c[21]; \
	(m).c[27].im = +5.000000000000000e-01*(h).c[26]; \
	(m).c[28].re = 0.0; \
	(m).c[28].im = -6.324555320336759e-01*(h).c[27]+1.290994448735805e-01*(h).c[34]; \
	(m).c[29].re = -5.000000000000000e-01*(h).c[28]; \
	(m).c[29].im = +5.000000000000000e-01*(h).c[33]; \
	(m).c[30].re = +5.000000000000000e-01*(h).c[4]; \
	(m).c[30].im = +5.000000000000000e-01*(h).c[29]; \
	(m).c[31].re = +5.000000000000000e-01*(h).c[10]; \
	(m).c[31].im = +5.000000000000000e-01*(h).c[30]; \
	(m).c[32].re = +5.000000000000000e-01*(h).c[16]; \
	(m).c[32].im = +5.000000000000000e-01*(h).c[31]; \
	(m).c[33].re = +5.000000000000000e-01*(h).c[22]; \
	(m).c[33].im = +5.000000000000000e-01*(h).c[32]; \
	(m).c[34].re = +5.000000000000000e-01*(h).c[28]; \
	(m).c[34].im = +5.000000000000000e-01*(h).c[33]; \
	(m).c[35].re = 0.0; \
	(m).c[35].im = -6.454972243679027e-01*(h).c[34];



 /*****
 *	h real 35-vector
 *	m COMPLEX matrix 6*6
 *	h(A) = -i*ReTr[rT(A).m]/C(r)
 */
#define _algebra_project(h,m) \
	(h).c[0] = -(m).c[1].re+(m).c[6].re; \
	(h).c[1] = -(-(m).c[12].re+(m).c[2].re); \
	(h).c[2] = -(-(m).c[18].re+(m).c[3].re); \
	(h).c[3] = -(-(m).c[24].re+(m).c[4].re); \
	(h).c[4] = -(-(m).c[30].re+(m).c[5].re); \
	(h).c[5] = +(m).c[1].im+(m).c[6].im; \
	(h).c[6] = -(-(m).c[0].im+(m).c[7].im); \
	(h).c[7] = -(-(m).c[13].re+(m).c[8].re); \
	(h).c[8] = -(-(m).c[19].re+(m).c[9].re); \
	(h).c[9] = -(m).c[10].re+(m).c[25].re; \
	(h).c[10] = -(m).c[11].re+(m).c[31].re; \
	(h).c[11] = +(m).c[12].im+(m).c[2].im; \
	(h).c[12] = +(m).c[13].im+(m).c[8].im; \
	(h).c[13] = +5.773502691896257e-01*(m).c[0].im-1.154700538379251e+00*(m).c[14].im+5.773502691896257e-01*(m).c[7].im; \
	(h).c[14] = -(m).c[15].re+(m).c[20].re; \
	(h).c[15] = -(m).c[16].re+(m).c[26].re; \
	(h).c[16] = -(m).c[17].re+(m).c[32].re; \
	(h).c[17] = +(m).c[18].im+(m).c[3].im; \
	(h).c[18] = +(m).c[19].im+(m).c[9].im; \
	(h).c[19] = +(m).c[15].im+(m).c[20].im; \
	(h).c[20] = +4.082482904638630e-01*(m).c[0].im+4.082482904638630e-01*(m).c[14].im-1.224744871391589e+00*(m).c[21].im+4.082482904638630e-01*(m).c[7].im; \
	(h).c[21] = -(m).c[22].re+(m).c[27].re; \
	(h).c[22] = -(m).c[23].re+(m).c[33].re; \
	(h).c[23] = +(m).c[24].im+(m).c[4].im; \
	(h).c[24] = +(m).c[10].im+(m).c[25].im; \
	(h).c[25] = +(m).c[16].im+(m).c[26].im; \
	(h).c[26] = +(m).c[22].im+(m).c[27].im; \
	(h).c[27] = +3.162277660168379e-01*(m).c[0].im+3.162277660168379e-01*(m).c[14].im+3.162277660168379e-01*(m).c[21].im-1.264911064067352e+00*(m).c[28].im+3.162277660168379e-01*(m).c[7].im; \
	(h).c[28] = -(m).c[29].re+(m).c[34].re; \
	(h).c[29] = +(m).c[30].im+(m).c[5].im; \
	(h).c[30] = +(m).c[11].im+(m).c[31].im; \
	(h).c[31] = +(m).c[17].im+(m).c[32].im; \
	(h).c[32] = +(m).c[23].im+(m).c[33].im; \
	(h).c[33] = +(m).c[29].im+(m).c[34].im; \
	(h).c[34] = +2.581988897471611e-01*(m).c[0].im+2.581988897471611e-01*(m).c[14].im+2.581988897471611e-01*(m).c[21].im+2.581988897471611e-01*(m).c[28].im-1.290994448735805e+00*(m).c[35].im+2.581988897471611e-01*(m).c[7].im;



 /*****
 *	m COMPLEX matrix 6*6
 *	h real 35-vector
 *	m = sum(A) i*h(A)*T(A)
 */
#define _fund_algebra_represent(m,h) \
	(m).c[0].re = 0.0; \
	(m).c[0].im = +2.886751345948129e-01*(h).c[13]+2.041241452319315e-01*(h).c[20]+1.581138830084190e-01*(h).c[27]+1.290994448735805e-01*(h).c[34]+5.000000000000000e-01*(h).c[6]; \
	(m).c[1].re = -5.000000000000000e-01*(h).c[0]; \
	(m).c[1].im = +5.000000000000000e-01*(h).c[5]; \
	(m).c[2].re = -5.000000000000000e-01*(h).c[1]; \
	(m).c[2].im = +5.000000000000000e-01*(h).c[11]; \
	(m).c[3].re = -5.000000000000000e-01*(h).c[2]; \
	(m).c[3].im = +5.000000000000000e-01*(h).c[17]; \
	(m).c[4].re = -5.000000000000000e-01*(h).c[3]; \
	(m).c[4].im = +5.000000000000000e-01*(h).c[23]; \
	(m).c[5].re = -5.000000000000000e-01*(h).c[4]; \
	(m).c[5].im = +5.000000000000000e-01*(h).c[29]; \
	(m).c[6].re = +5.000000000000000e-01*(h).c[0]; \
	(m).c[6].im = +5.000000000000000e-01*(h).c[5]; \
	(m).c[7].re = 0.0; \
	(m).c[7].im = +2.886751345948129e-01*(h).c[13]+2.041241452319315e-01*(h).c[20]+1.581138830084190e-01*(h).c[27]+1.290994448735805e-01*(h).c[34]-5.000000000000000e-01*(h).c[6]; \
	(m).c[8].re = -5.000000000000000e-01*(h).c[7]; \
	(m).c[8].im = +5.000000000000000e-01*(h).c[12]; \
	(m).c[9].re = -5.000000000000000e-01*(h).c[8]; \
	(m).c[9].im = +5.000000000000000e-01*(h).c[18]; \
	(m).c[10].re = -5.000000000000000e-01*(h).c[9]; \
	(m).c[10].im = +5.000000000000000e-01*(h).c[24]; \
	(m).c[11].re = -5.000000000000000e-01*(h).c[10]; \
	(m).c[11].im = +5.000000000000000e-01*(h).c[30]; \
	(m).c[12].re = +5.000000000000000e-01*(h).c[1]; \
	(m).c[12].im = +5.000000000000000e-01*(h).c[11]; \
	(m).c[13].re = +5.000000000000000e-01*(h).c[7]; \
	(m).c[13].im = +5.000000000000000e-01*(h).c[12]; \
	(m).c[14].re = 0.0; \
	(m).c[14].im = -5.773502691896257e-01*(h).c[13]+2.041241452319315e-01*(h).c[20]+1.581138830084190e-01*(h).c[27]+1.290994448735805e-01*(h).c[34]; \
	(m).c[15].re = -5.000000000000000e-01*(h).c[14]; \
	(m).c[15].im = +5.000000000000000e-01*(h).c[19]; \
	(m).c[16].re = -5.000000000000000e-01*(h).c[15]; \
	(m).c[16].im = +5.000000000000000e-01*(h).c[25]; \
	(m).c[17].re = -5.000000000000000e-01*(h).c[16]; \
	(m).c[17].im = +5.000000000000000e-01*(h).c[31]; \
	(m).c[18].re = +5.000000000000000e-01*(h).c[2]; \
	(m).c[18].im = +5.000000000000000e-01*(h).c[17]; \
	(m).c[19].re = +5.000000000000000e-01*(h).c[8]; \
	(m).c[19].im = +5.000000000000000e-01*(h).c[18]; \
	(m).c[20].re = +5.000000000000000e-01*(h).c[14]; \
	(m).c[20].im = +5.000000000000000e-01*(h).c[19]; \
	(m).c[21].re = 0.0; \
	(m).c[21].im = -6.123724356957945e-01*(h).c[20]+1.581138830084190e-01*(h).c[27]+1.290994448735805e-01*(h).c[34]; \
	(m).c[22].re = -5.000000000000000e-01*(h).c[21]; \
	(m).c[22].im = +5.000000000000000e-01*(h).c[26]; \
	(m).c[23].re = -5.000000000000000e-01*(h).c[22]; \
	(m).c[23].im = +5.000000000000000e-01*(h).c[32]; \
	(m).c[24].re = +5.000000000000000e-01*(h).c[3]; \
	(m).c[24].im = +5.000000000000000e-01*(h).c[23]; \
	(m).c[25].re = +5.000000000000000e-01*(h).c[9]; \
	(m).c[25].im = +5.000000000000000e-01*(h).c[24]; \
	(m).c[26].re = +5.000000000000000e-01*(h).c[15]; \
	(m).c[26].im = +5.000000000000000e-01*(h).c[25]; \
	(m).c[27].re = +5.000000000000000e-01*(h).c[21]; \
	(m).c[27].im = +5.000000000000000e-01*(h).c[26]; \
	(m).c[28].re = 0.0; \
	(m).c[28].im = -6.324555320336759e-01*(h).c[27]+1.290994448735805e-01*(h).c[34]; \
	(m).c[29].re = -5.000000000000000e-01*(h).c[28]; \
	(m).c[29].im = +5.000000000000000e-01*(h).c[33]; \
	(m).c[30].re = +5.000000000000000e-01*(h).c[4]; \
	(m).c[30].im = +5.000000000000000e-01*(h).c[29]; \
	(m).c[31].re = +5.000000000000000e-01*(h).c[10]; \
	(m).c[31].im = +5.000000000000000e-01*(h).c[30]; \
	(m).c[32].re = +5.000000000000000e-01*(h).c[16]; \
	(m).c[32].im = +5.000000000000000e-01*(h).c[31]; \
	(m).c[33].re = +5.000000000000000e-01*(h).c[22]; \
	(m).c[33].im = +5.000000000000000e-01*(h).c[32]; \
	(m).c[34].re = +5.000000000000000e-01*(h).c[28]; \
	(m).c[34].im = +5.000000000000000e-01*(h).c[33]; \
	(m).c[35].re = 0.0; \
	(m).c[35].im = -6.454972243679027e-01*(h).c[34];



 /*****
 *	h real 35-vector
 *	m COMPLEX matrix 6*6
 *	h(A) = -i*ReTr[T(A).m]/C(FUND)
 */
#define _fund_algebra_project(h,m) \
	(h).c[0] = -(m).c[1].re+(m).c[6].re; \
	(h).c[1] = -(-(m).c[12].re+(m).c[2].re); \
	(h).c[2] = -(-(m).c[18].re+(m).c[3].re); \
	(h).c[3] = -(-(m).c[24].re+(m).c[4].re); \
	(h).c[4] = -(-(m).c[30].re+(m).c[5].re); \
	(h).c[5] = +(m).c[1].im+(m).c[6].im; \
	(h).c[6] = -(-(m).c[0].im+(m).c[7].im); \
	(h).c[7] = -(-(m).c[13].re+(m).c[8].re); \
	(h).c[8] = -(-(m).c[19].re+(m).c[9].re); \
	(h).c[9] = -(m).c[10].re+(m).c[25].re; \
	(h).c[10] = -(m).c[11].re+(m).c[31].re; \
	(h).c[11] = +(m).c[12].im+(m).c[2].im; \
	(h).c[12] = +(m).c[13].im+(m).c[8].im; \
	(h).c[13] = +5.773502691896257e-01*(m).c[0].im-1.154700538379251e+00*(m).c[14].im+5.773502691896257e-01*(m).c[7].im; \
	(h).c[14] = -(m).c[15].re+(m).c[20].re; \
	(h).c[15] = -(m).c[16].re+(m).c[26].re; \
	(h).c[16] = -(m).c[17].re+(m).c[32].re; \
	(h).c[17] = +(m).c[18].im+(m).c[3].im; \
	(h).c[18] = +(m).c[19].im+(m).c[9].im; \
	(h).c[19] = +(m).c[15].im+(m).c[20].im; \
	(h).c[20] = +4.082482904638630e-01*(m).c[0].im+4.082482904638630e-01*(m).c[14].im-1.224744871391589e+00*(m).c[21].im+4.082482904638630e-01*(m).c[7].im; \
	(h).c[21] = -(m).c[22].re+(m).c[27].re; \
	(h).c[22] = -(m).c[23].re+(m).c[33].re; \
	(h).c[23] = +(m).c[24].im+(m).c[4].im; \
	(h).c[24] = +(m).c[10].im+(m).c[25].im; \
	(h).c[25] = +(m).c[16].im+(m).c[26].im; \
	(h).c[26] = +(m).c[22].im+(m).c[27].im; \
	(h).c[27] = +3.162277660168379e-01*(m).c[0].im+3.162277660168379e-01*(m).c[14].im+3.162277660168379e-01*(m).c[21].im-1.264911064067352e+00*(m).c[28].im+3.162277660168379e-01*(m).c[7].im; \
	(h).c[28] = -(m).c[29].re+(m).c[34].re; \
	(h).c[29] = +(m).c[30].im+(m).c[5].im; \
	(h).c[30] = +(m).c[11].im+(m).c[31].im; \
	(h).c[31] = +(m).c[17].im+(m).c[32].im; \
	(h).c[32] = +(m).c[23].im+(m).c[33].im; \
	(h).c[33] = +(m).c[29].im+(m).c[34].im; \
	(h).c[34] = +2.581988897471611e-01*(m).c[0].im+2.581988897471611e-01*(m).c[14].im+2.581988897471611e-01*(m).c[21].im+2.581988897471611e-01*(m).c[28].im-1.290994448735805e+00*(m).c[35].im+2.581988897471611e-01*(m).c[7].im;


#endif 
